/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...

*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_WORLD_ENTITY

#include "spacecraft_2d.h"

#include "weapons/weapon_manager.h"
#include "weapons/test_gun.h"
#include "weapons/turret.h"
#include "weapons/cannon.h"

#include "util/loading/factory.h"
#include "key_mapper.h"
#include "state.h"

#include "graphics_engine.h"
#include "dot_emitter.h"
#include "sprite_particles.h"

#include "debug.h"

CREATE_FACTORY(Spacecraft2D, CL_SPACECRAFT_2D);


/**
 * @brief loads a Spacecraft2D information from a specified file.
 * @param fileName the name of the File to load the spacecraft_2d from (absolute path)
 */
Spacecraft2D::Spacecraft2D(const std::string& fileName)
{
  this->init();
  TiXmlDocument doc(fileName);

  if(!doc.LoadFile())
  {
    PRINTF(2)("Loading file %s failed for Spacecraft2D.\n", fileName.c_str());
    return;
  }

  this->loadParams(doc.RootElement());
}

/**
 * @brief creates a new Spaceship from Xml Data
 * @param root the xml element containing spaceship data

   @todo add more parameters to load
*/
Spacecraft2D::Spacecraft2D(const TiXmlElement* root)
{
  this->init();
  if (root != NULL)
    this->loadParams(root);

  //weapons:
  Weapon* wpRight = new TestGun(0);
  wpRight->setName("testGun Right");
  Weapon* wpLeft = new TestGun(1);
  wpLeft->setName("testGun Left");
  //Weapon* cannon = dynamic_cast<Weapon*>(Factory::fabricate(CL_HYPERBLASTER));

  //  cannon->setName("BFG");

  this->addWeapon(wpLeft, 1, 0);
  this->addWeapon(wpRight,1 ,1);
  //this->addWeapon(cannon, 0, 2);

  this->getWeaponManager().changeWeaponConfig(1);
  dynamic_cast<Element2D*>(this->getWeaponManager().getFixedTarget())->setVisibility( false);
}


/**
 *  destructs the spacecraft_2d, deletes alocated memory
 */
Spacecraft2D::~Spacecraft2D ()
{
  this->setPlayer(NULL);
}

/**
 * @brief initializes a Spacecraft2D
 */
void Spacecraft2D::init()
{
  //  this->setRelDir(Quaternion(M_PI, Vector(1,0,0)));
  this->setClassID(CL_SPACECRAFT_2D, "Spacecraft2D");

  this->setSupportedPlaymodes(Playable::Full3D | Playable::Horizontal);


  bForward = bBackward = bLeft = bRight = false;
  mouseSensitivity = 0.005;

  this->cameraLook = 0.0f;
  this->rotation = 0.0f;
  this->acceleration = 10.0f;
  this->altitude = 0.0f;

  this->setHealthMax(100);
  this->setHealth(100);


  // camera - issue
  this->travelNode.addNodeFlags(PNODE_PROHIBIT_CHILD_DELETE);
  //this->cameraNode.setParentMode(PNODE_ROTATE_MOVEMENT);
  //this->cameraNode.setParent(this);

  // rotors
  // PARTICLES
  this->burstEmitter = new DotEmitter(200, 5.0, .01);
  this->burstEmitter->setParent(this);
  this->burstEmitter->setRelCoor(0, -0.7, 0);
  this->burstEmitter->setRelDir(Quaternion(-M_PI_2, Vector(0,0,1)));
  this->burstEmitter->setName("Spacecraft2D_Burst_emitter_Left");


  this->burstSystem = new SpriteParticles(1000);
  this->burstSystem->addEmitter(this->burstEmitter);
  this->burstSystem->setName("SpaceShip_Burst_System");
  ((SpriteParticles*)this->burstSystem)->setMaterialTexture("maps/radial-trans-noise.png");
  this->burstSystem->setLifeSpan(1.0, .3);
  this->burstSystem->setRadius(0.0, 1.5);
  this->burstSystem->setRadius(0.05, 1.8);
  this->burstSystem->setRadius(.5, .8);
  this->burstSystem->setRadius(1.0, 0);
  this->burstSystem->setColor(0.0, .7,.7,1,.5);
  this->burstSystem->setColor(0.2, 0,0,0.8,.5);
  this->burstSystem->setColor(0.5, .5,.5,.8,.3);
  this->burstSystem->setColor(1.0, .8,.8,.8,.0);


  //add events to the eventlist of the Playable
  this->registerEvent(KeyMapper::PEV_FORWARD);
  this->registerEvent(KeyMapper::PEV_BACKWARD);
  this->registerEvent(KeyMapper::PEV_LEFT);
  this->registerEvent(KeyMapper::PEV_RIGHT);
  this->registerEvent(KeyMapper::PEV_UP);
  this->registerEvent(KeyMapper::PEV_DOWN);
  this->registerEvent(KeyMapper::PEV_FIRE1);
  this->registerEvent(KeyMapper::PEV_NEXT_WEAPON);
  this->registerEvent(KeyMapper::PEV_PREVIOUS_WEAPON);
  this->registerEvent(EV_MOUSE_MOTION);

  dynamic_cast<Element2D*>(this->getWeaponManager().getFixedTarget())->setVisibility( false);

  // WEAPON_MANAGER configuration
  this->getWeaponManager().setSlotCount(5);

  this->getWeaponManager().setSlotPosition(0, Vector(-0.28, 1.186, -2.750));
  this->getWeaponManager().setSlotCapability(0, WTYPE_ALLDIRS | WTYPE_DIRECTIONAL);

  this->getWeaponManager().setSlotPosition(1, Vector(-0.28, 1.186, 2.750));
  this->getWeaponManager().setSlotCapability(1, WTYPE_ALLDIRS | WTYPE_DIRECTIONAL);

  this->getWeaponManager().setSlotPosition(2, Vector(-1.63, .809, -.003));
  this->getWeaponManager().setSlotCapability(2, WTYPE_HEAVY);

  /// TODO: THESE ARE TOO MUCH
  this->getWeaponManager().setSlotPosition(3, Vector(-1.63, .678, -.652));
  this->getWeaponManager().setSlotDirection(3, Quaternion(-24/180 * M_PI, Vector(1,0,0)));

  this->getWeaponManager().setSlotPosition(4, Vector(-1.63, .678, .652));
  this->getWeaponManager().setSlotDirection(4, Quaternion(24/180 * M_PI, Vector(1,0,0)));

  this->travelNode.setRelCoor(0,0,0);
  //this->getWeaponManager().getFixedTarget()->setParent(&this->cameraNode);
  //this->getWeaponManager().getFixedTarget()->setRelCoor(1000,0,0);

  registerVar( new SynchronizeableBool( &bForward, &bForward, "bForward", PERMISSION_OWNER ) );
  registerVar( new SynchronizeableBool( &bBackward, &bBackward, "bBackward", PERMISSION_OWNER ) );
  registerVar( new SynchronizeableBool( &bLeft, &bLeft, "bLeft", PERMISSION_OWNER ) );
  registerVar( new SynchronizeableBool( &bRight, &bRight, "bRight", PERMISSION_OWNER ) );
  //registerVar( new SynchronizeableQuaternion( &direction, &direction, "direction", PERMISSION_OWNER ) );
  registerVar( new SynchronizeableFloat( &cameraLook, &cameraLook, "cameraLook", PERMISSION_OWNER ) );
  registerVar( new SynchronizeableFloat( &rotation, &rotation, "rotation", PERMISSION_OWNER ) );
}

/**
 * @brief loads the Settings of a Spacecraft2D from an XML-element.
 * @param root the XML-element to load the Spaceship's properties from
 */
void Spacecraft2D::loadParams(const TiXmlElement* root)
{
  Playable::loadParams(root);
}

void Spacecraft2D::setPlayDirection(const Quaternion& rot, float speed)
{
  this->direction = Quaternion (rot.getHeading(), Vector(0,1,0));
}

void Spacecraft2D::setTravelDirecton(const Quaternion& rot, float speed)
{
  this->setPlayDirection(rot, speed);
}

void Spacecraft2D::setTravelSpeed(float travelSpeed)
{
  this->travelSpeed = travelSpeed;
}



void Spacecraft2D::enter()
{
  dynamic_cast<Element2D*>(this->getWeaponManager().getFixedTarget())->setVisibility( true);

  if (State::getCameraNode != NULL)
  {
    State::getCameraNode()->setParentSoft(&this->travelNode);
    State::getCameraNode()->setRelCoorSoft(-10, 50,0);
    State::getCameraTargetNode()->setParentSoft(&this->travelNode);
  }
}

void Spacecraft2D::leave()
{
  dynamic_cast<Element2D*>(this->getWeaponManager().getFixedTarget())->setVisibility( false);
  this->detachCamera();

}


/**
 * @brief effect that occurs after the Spacecraft2D is spawned
*/
void Spacecraft2D::postSpawn ()
{
  //setCollision(new CollisionCluster(1.0, Vector(0,0,0)));
}

/**
 * @brief the action occuring if the spacecraft_2d left the game
*/
void Spacecraft2D::leftWorld ()
{}

/**
 * @brief this function is called, when two entities collide
 * @param entity: the world entity with whom it collides
 *
 * Implement behaviour like damage application or other miscellaneous collision stuff in this function
 */
void Spacecraft2D::collidesWith(WorldEntity* entity, const Vector& location)
{
  Playable::collidesWith(entity, location);
}



/**
 * @brief the function called for each passing timeSnap
 * @param time The timespan passed since last update
*/
void Spacecraft2D::tick (float dt)
{
  //  this->debugNode(1);
  Playable::tick(dt);

  // spaceship controlled movement
  this->movement(dt);

  // TRYING TO FIX PNode.
  //this->travelNode.setAbsCoorSoft(this->getAbsCoor() + Vector(0.0f, 5.0f, 0.0f), 30.0f);
  //this->travelNode.setRelDirSoft(this->getAbsDir(), 30.0f);
}

/**
 * @brief calculate the velocity
 * @param time the timeslice since the last frame
*/
void Spacecraft2D::movement (float dt)
{
  Vector accel(0.0, 0.0, 0.0);

  if( this->bForward )
  {
    accel += Vector(this->acceleration, 0, 0);
  }

  if( this->bBackward )
  {
    accel -= Vector(this->acceleration, 0, 0);
  }
  if( this->bLeft)
  {
    accel -= Vector(0, 0, this->acceleration);
  }

  if( this->bRight)
  {
    accel += Vector(0, 0, this->acceleration);
  }


  switch(this->getPlaymode())
  {
    case Playable::Full3D:
      {
        Vector accelerationDir = this->getAbsDir().apply(accel * this->acceleration);

        // this is the air friction (necessary for a smooth control)

        this->velocity += (accelerationDir)* dt;
        this->shiftCoor (this->velocity * dt);

        // limit the maximum rotation speed.
        if (this->rotation != 0.0f)
        {
          float maxRot = 10.0 * dt;
          if (unlikely(this->rotation > maxRot)) this->rotation = maxRot;
          if (unlikely(this->rotation < -maxRot)) this->rotation = -maxRot;
          this->direction *= Quaternion(-M_PI/4.0*this->rotation, Vector(0,1,0));

          this->rotation = 0.0f;
        }

        this->setRelDirSoft(this->direction * Quaternion(-cameraLook, Vector(0,0,1)), 5);

      }
      break;

    case Playable::Horizontal:
      {
        accel.y = 0.0;
        Vector accelerationDir = this->getAbsDir().apply(accel * this->acceleration);
        accelerationDir.y = 0.0;

        // this is the air friction (necessary for a smooth control)


        this->velocity += (accelerationDir )* dt;
        this->shiftCoor (this->velocity * dt);

        // limit the maximum rotation speed.
        if (this->rotation != 0.0f)
        {
          float maxRot = 10.0 * dt;
          if (unlikely(this->rotation > 0.01 || this->rotation < 0.01)) this->rotation /=1.5;
          this->direction *= Quaternion(-M_PI/4.0*this->rotation, Vector(0,1,0));

          this->rotation = 0.0f;
        }

        this->setRelDirSoft(this->direction, 5);
      }
      break;

    default:
      PRINTF(2)("Playmode %s Not Implemented\n", Playable::playmodeToString(this->getPlaymode()).c_str());
  }
}


void Spacecraft2D::draw() const
{
  WorldEntity::draw();
}

/**
 * @todo switch statement ??
 */
void Spacecraft2D::process(const Event &event)
{
  Playable::process(event);

  if( event.type == KeyMapper::PEV_LEFT)
    this->bLeft = event.bPressed;
  else if( event.type == KeyMapper::PEV_RIGHT)
    this->bRight = event.bPressed;
  else if( event.type == KeyMapper::PEV_FORWARD)
    this->bForward = event.bPressed; //this->shiftCoor(0,.1,0);
  else if( event.type == KeyMapper::PEV_BACKWARD)
    this->bBackward = event.bPressed; //this->shiftCoor(0,-.1,0);
  else if( event.type == EV_MOUSE_MOTION)
  {
    float xMouse, yMouse;
    xMouse = event.xRel*mouseSensitivity;
    yMouse = event.yRel*mouseSensitivity;

    // rotate the Player around the y-axis
    if (this->getPlaymode() == Playable::Full3D)
      this->rotation += xMouse;

    this->cameraLook += yMouse;
    // rotate the Camera around the z-axis
    if (cameraLook > M_PI_4)
      cameraLook = M_PI_4;
    else if (cameraLook < -M_PI_4)
      cameraLook = -M_PI_4;
    //this->cameraNode.setRelDirSoft(this->direction,10);
  }
}
